@@ -4,7 +4,10 @@ import warnings
 
 import numpy as np
 from numpy.testing import (assert_almost_equal, assert_equal, assert_allclose,
-                           assert_, assert_raises, TestCase, run_module_suite)
+                           assert_)
+from pytest import raises as assert_raises
+
+from scipy._lib._numpy_compat import suppress_warnings
 from scipy.signal import (ss2tf, tf2ss, lsim2, impulse2, step2, lti,
                           dlti, bode, freqresp, lsim, impulse, step,
                           abcd_normalize, place_poles,
@@ -34,7 +37,7 @@ def _assert_poles_close(P1,P2, rtol=1e-8, atol=1e-8):
             raise ValueError("Can't find pole " + str(p1) + " in " + str(P2))
 
 
-class TestPlacePoles(TestCase):
+class TestPlacePoles(object):
 
     def _check(self, A, B, P, **kwargs):
         """
@@ -65,7 +68,10 @@ class TestPlacePoles(TestCase):
         # values are almost equal. This is to improve code coverage but I
         # have no way to be sure this code is really reached
 
-        self._check(A, B, (2,2,3,3))
+        # on some architectures this can lead to a RuntimeWarning invalid
+        # value in divide (see gh-7590), so suppress it for now
+        with np.errstate(invalid='ignore'):
+            self._check(A, B, (2,2,3,3))
 
     def test_complex(self):
         # Test complex pole placement on a linearized car model, taken from L.
@@ -213,6 +219,7 @@ class TestPlacePoles(TestCase):
         # Should not raise ValueError as the poles can be placed but should
         # raise a warning as the convergence is not reached
         with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
             fsf = place_poles(A, B, (-1,-2,-3,-4), rtol=1e-16, maxiter=42)
             assert_(len(w) == 1)
             assert_(issubclass(w[-1].category, UserWarning))
@@ -236,7 +243,7 @@ class TestPlacePoles(TestCase):
 
 class TestSS2TF:
 
-    def tst_matrix_shapes(self, p, q, r):
+    def check_matrix_shapes(self, p, q, r):
         ss2tf(np.zeros((p, p)),
               np.zeros((p, q)),
               np.zeros((r, p)),
@@ -246,7 +253,7 @@ class TestSS2TF:
         # Each tuple holds:
         #   number of states, number of inputs, number of outputs
         for p, q, r in [(3, 3, 3), (1, 3, 3), (1, 1, 1)]:
-            yield self.tst_matrix_shapes, p, q, r
+            self.check_matrix_shapes(p, q, r)
 
     def test_basic(self):
         # Test a round trip through tf2ss and ss2tf.
@@ -385,8 +392,8 @@ class TestSS2TF:
 
 class TestLsim(object):
     def lti_nowarn(self, *args):
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", BadCoefficients)
+        with suppress_warnings() as sup:
+            sup.filter(BadCoefficients)
             system = lti(*args)
         return system
 
@@ -522,8 +529,8 @@ class Test_lsim2(object):
         D = np.zeros((1, 2))
 
         t = np.linspace(0, 10.0, 101)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", BadCoefficients)
+        with suppress_warnings() as sup:
+            sup.filter(BadCoefficients)
             tout, y, x = lsim2((A,B,C,D), T=t, X0=[1.0, 1.0])
         expected_y = np.exp(-tout)
         expected_x0 = np.exp(-tout)
@@ -617,12 +624,12 @@ class _TestImpulseFuncs(object):
 
 
 class TestImpulse2(_TestImpulseFuncs):
-    def setup(self):
+    def setup_method(self):
         self.func = impulse2
 
 
 class TestImpulse(_TestImpulseFuncs):
-    def setup(self):
+    def setup_method(self):
         self.func = impulse
 
 
@@ -694,7 +701,7 @@ class _TestStepFuncs(object):
 
 
 class TestStep2(_TestStepFuncs):
-    def setup(self):
+    def setup_method(self):
         self.func = step2
 
     def test_05(self):
@@ -711,7 +718,7 @@ class TestStep2(_TestStepFuncs):
 
 
 class TestStep(_TestStepFuncs):
-    def setup(self):
+    def setup_method(self):
         self.func = step
 
     def test_complex_input(self):
@@ -757,13 +764,6 @@ class TestStateSpace(object):
         s = StateSpace(np.array([[1, 2], [3, 4]]), np.array([[1], [2]]),
                        np.array([[1, 0]]), np.array([[0]]))
 
-    def _compare_systems(self, sys1, sys2):
-        # Compare the contents of two systems
-        assert_equal(sys1.A, sys2.A)
-        assert_equal(sys1.B, sys2.B)
-        assert_equal(sys1.C, sys2.C)
-        assert_equal(sys1.D, sys2.D)
-
     def test_conversion(self):
         # Check the conversion functions
         s = StateSpace(1, 2, 3, 4)
@@ -781,31 +781,9 @@ class TestStateSpace(object):
 
         # Getters
         s = StateSpace(1, 1, 1, 1)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            assert_equal(s.num, [1, 0])
-            assert_equal(s.den, [1, -1])
-            assert_equal(s.poles, [1])
-            assert_equal(s.zeros, [0])
-            assert_equal(s.gain, 1)
-
-        # transfer function setters
-        s2 = StateSpace(2, 2, 2, 2)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            s2.num = [1, 0]
-            s2.den = [1, -1]
-            self._compare_systems(s, s2)
-
-        # zpk setters
-        s2 = StateSpace(2, 2, 2, 2)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            assert_(s.dt is None)
-            s2.poles = 1
-            s2.zeros = 0
-            s2.gain = 1
-            self._compare_systems(s, s2)
+        assert_equal(s.poles, [1])
+        assert_equal(s.zeros, [0])
+        assert_(s.dt is None)
 
 
 class TestTransferFunction(object):
@@ -815,11 +793,6 @@ class TestTransferFunction(object):
         s = TransferFunction([1], [2])
         s = TransferFunction(np.array([1]), np.array([2]))
 
-    def _compare_systems(self, sys1, sys2):
-        # Compare the contents of two systems
-        assert_equal(sys1.num, sys2.num)
-        assert_equal(sys1.den, sys2.den)
-
     def test_conversion(self):
         # Check the conversion functions
         s = TransferFunction([1, 0], [1, -1])
@@ -837,34 +810,8 @@ class TestTransferFunction(object):
 
         # Getters
         s = TransferFunction([1, 0], [1, -1])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            assert_equal(s.poles, [1])
-            assert_equal(s.zeros, [0])
-            assert_equal(s.gain, 1)
-            assert_equal(s.A, 1)
-            assert_equal(s.B, 1)
-            assert_equal(s.C, 1)
-            assert_equal(s.D, 1)
-
-        # state space setters
-        s2 = TransferFunction([2, 3], [4, 5])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            s2.A = 1
-            s2.B = 1
-            s2.C = 1
-            s2.D = 1
-            self._compare_systems(s, s2)
-
-        # zpk setters
-        s2 = TransferFunction([2, 3], [4, 5])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            s2.poles = 1
-            s2.zeros = 0
-            s2.gain = 1
-            self._compare_systems(s, s2)
+        assert_equal(s.poles, [1])
+        assert_equal(s.zeros, [0])
 
 
 class TestZerosPolesGain(object):
@@ -874,14 +821,6 @@ class TestZerosPolesGain(object):
         s = ZerosPolesGain([1], [2], 1)
         s = ZerosPolesGain(np.array([1]), np.array([2]), 1)
 
-    def _compare_systems(self, sys1, sys2):
-        #Compare the contents of two systems
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            assert_equal(sys1.poles, sys2.poles)
-            assert_equal(sys1.zeros, sys2.zeros)
-            assert_equal(sys1.gain, sys2.gain)
-
     def test_conversion(self):
         #Check the conversion functions
         s = ZerosPolesGain(1, 2, 3)
@@ -893,42 +832,9 @@ class TestZerosPolesGain(object):
         assert_(ZerosPolesGain(s) is not s)
         assert_(s.to_zpk() is not s)
 
-    def test_properties(self):
-        # Test setters/getters for cross class properties.
-        # This implicitly tests to_ss() and to_tf()
-
-        # Getters
-        s = ZerosPolesGain(0, 1, 1)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            assert_equal(s.num, [1, 0])
-            assert_equal(s.den, [1, -1])
-            assert_equal(s.A, 1)
-            assert_equal(s.B, 1)
-            assert_equal(s.C, 1)
-            assert_equal(s.D, 1)
-
-        # state space setters
-        s2 = ZerosPolesGain([2], [6], 3)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            s2.A = 1
-            s2.B = 1
-            s2.C = 1
-            s2.D = 1
-            self._compare_systems(s, s2)
-
-        # tf setters
-        s2 = ZerosPolesGain([2], [5], 3)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-            s2.num = [1, 0]
-            s2.den = [1, -1]
-            self._compare_systems(s, s2)
-
 
 class Test_abcd_normalize(object):
-    def setup(self):
+    def setup_method(self):
         self.A = np.array([[1.0, 2.0], [3.0, 4.0]])
         self.B = np.array([[-1.0], [5.0]])
         self.C = np.array([[4.0, 5.0]])
@@ -1152,8 +1058,8 @@ class Test_bode(object):
         B = np.array([[0.0], [0.0], [1.0]])
         C = np.array([[1.0, 0.0, 0.0]])
         D = np.array([[0.0]])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", BadCoefficients)
+        with suppress_warnings() as sup:
+            sup.filter(BadCoefficients)
             system = lti(A, B, C, D)
             w, mag, phase = bode(system, n=100)
 
@@ -1163,8 +1069,8 @@ class Test_bode(object):
 
 class Test_freqresp(object):
 
-    def test_real_part_manual(self):
-        # Test freqresp() real part calculation (manual sanity check).
+    def test_output_manual(self):
+        # Test freqresp() output calculation (manual sanity check).
         # 1st order low-pass filter: H(s) = 1 / (s + 1),
         #   re(H(s=0.1)) ~= 0.99
         #   re(H(s=1)) ~= 0.5
@@ -1173,41 +1079,20 @@ class Test_freqresp(object):
         w = [0.1, 1, 10]
         w, H = freqresp(system, w=w)
         expected_re = [0.99, 0.5, 0.0099]
-        assert_almost_equal(H.real, expected_re, decimal=1)
-
-    def test_imag_part_manual(self):
-        # Test freqresp() imaginary part calculation (manual sanity check).
-        # 1st order low-pass filter: H(s) = 1 / (s + 1),
-        #   im(H(s=0.1)) ~= -0.099
-        #   im(H(s=1)) ~= -0.5
-        #   im(H(s=10)) ~= -0.099
-        system = lti([1], [1, 1])
-        w = [0.1, 1, 10]
-        w, H = freqresp(system, w=w)
         expected_im = [-0.099, -0.5, -0.099]
+        assert_almost_equal(H.real, expected_re, decimal=1)
         assert_almost_equal(H.imag, expected_im, decimal=1)
 
-    def test_real_part(self):
-        # Test freqresp() real part calculation.
+    def test_output(self):
+        # Test freqresp() output calculation.
         # 1st order low-pass filter: H(s) = 1 / (s + 1)
         system = lti([1], [1, 1])
         w = [0.1, 1, 10, 100]
         w, H = freqresp(system, w=w)
-        jw = w * 1j
-        y = np.polyval(system.num, jw) / np.polyval(system.den, jw)
-        expected_re = y.real
-        assert_almost_equal(H.real, expected_re)
-
-    def test_imag_part(self):
-        # Test freqresp() imaginary part calculation.
-        # 1st order low-pass filter: H(s) = 1 / (s + 1)
-        system = lti([1], [1, 1])
-        w = [0.1, 1, 10, 100]
-        w, H = freqresp(system, w=w)
-        jw = w * 1j
-        y = np.polyval(system.num, jw) / np.polyval(system.den, jw)
-        expected_im = y.imag
-        assert_almost_equal(H.imag, expected_im)
+        s = w * 1j
+        expected = np.polyval(system.num, s) / np.polyval(system.den, s)
+        assert_almost_equal(H.real, expected.real)
+        assert_almost_equal(H.imag, expected.imag)
 
     def test_freq_range(self):
         # Test that freqresp() finds a reasonable frequency range.
@@ -1238,13 +1123,22 @@ class Test_freqresp(object):
         B = np.array([[0.0],[0.0],[1.0]])
         C = np.array([[1.0, 0.0, 0.0]])
         D = np.array([[0.0]])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", BadCoefficients)
+        with suppress_warnings() as sup:
+            sup.filter(BadCoefficients)
             system = lti(A, B, C, D)
             w, H = freqresp(system, n=100)
-        expected_magnitude = np.sqrt(1.0 / (1.0 + w**6))
-        assert_almost_equal(np.abs(H), expected_magnitude)
+        s = w * 1j
+        expected = (1.0 / (1.0 + 2*s + 2*s**2 + s**3))
+        assert_almost_equal(H.real, expected.real)
+        assert_almost_equal(H.imag, expected.imag)
 
+    def test_from_zpk(self):
+        # 4th order low-pass filter: H(s) = 1 / (s + 1)
+        system = lti([],[-1]*4,[1])
+        w = [0.1, 1, 10, 100]
+        w, H = freqresp(system, w=w)
+        s = w * 1j
+        expected = 1 / (s + 1)**4
+        assert_almost_equal(H.real, expected.real)
+        assert_almost_equal(H.imag, expected.imag)
 
-if __name__ == "__main__":
-    run_module_suite()
